@@ -84,7 +84,6 @@ struct option lopts[] = {
 	{"oneshot", 0, NULL, 'o'},
 	{"debug", 0, NULL, 'd'},
 	{"foreground", 0, NULL, 'f'},
-	{"hintpolicy", 1, NULL, 'h'},
 	{"powerthresh", 1, NULL, 'p'},
 	{"banirq", 1 , NULL, 'i'},
 	{"deepestcache", 1, NULL, 'c'},
@@ -99,7 +98,7 @@ struct option lopts[] = {
 
 static void usage(void)
 {
-	log(TO_CONSOLE, LOG_INFO, "irqbalance [--oneshot | -o] [--debug | -d] [--foreground | -f] [--journal | -j] [--hintpolicy= | -h [exact|subset|ignore]]\n");
+	log(TO_CONSOLE, LOG_INFO, "irqbalance [--oneshot | -o] [--debug | -d] [--foreground | -f] [--journal | -j]\n");
 	log(TO_CONSOLE, LOG_INFO, "	[--powerthresh= | -p <off> | <n>] [--banirq= | -i <n>] [--banmod= | -m <module>] [--policyscript= | -l <script>]\n");
 	log(TO_CONSOLE, LOG_INFO, "	[--pid= | -s <file>] [--deepestcache= | -c <n>] [--interval= | -t <n>]\n");
 }
@@ -376,7 +375,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 		}
 		if ((recv_size = recvmsg(sock, &msg, 0)) < 0) {
 			log(TO_ALL, LOG_WARNING, "Error while receiving data.\n");
-			goto out;
+			goto out_close;
 		}
 		cmsg = CMSG_FIRSTHDR(&msg);
 		if ((cmsg->cmsg_level == SOL_SOCKET) &&
@@ -388,7 +387,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 		}
 		if (!valid_user) {
 			log(TO_ALL, LOG_INFO, "Permission denied for user to connect to socket.\n");
-			goto out;
+			goto out_close;
 		}
 
 		if (!strncmp(buff, "stats", strlen("stats"))) {
@@ -421,7 +420,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 				need_rescan = 1;
 				if (!strncmp(irq_string, "NONE", strlen("NONE"))) {
 					free(irq_string);
-					goto out;
+					goto out_close;
 				}
 				int irq = strtoul(irq_string, &end, 10);
 				do {
@@ -457,6 +456,7 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 			free(setup);
 		}
 
+out_close:
 		close(sock);
 	}
 
@@ -482,7 +482,7 @@ int init_socket()
 	 */
 	addr.sun_family = AF_UNIX;
 	snprintf(socket_name, 64, "%s/%s%d.sock", SOCKET_TMPFS, SOCKET_PATH, getpid());
-	strncpy(addr.sun_path, socket_name, strlen(socket_name));
+	strncpy(addr.sun_path, socket_name, sizeof(addr.sun_path));
 	if (bind(socket_fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
 		log(TO_ALL, LOG_WARNING, "Daemon couldn't be bound to the file-based socket.\n");
 
